@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf8 -*-
 ################################################################################
 #
@@ -30,7 +29,7 @@
 # 
 ################################################################################
 
-"""
+u"""
 
 How to use::
 
@@ -40,25 +39,23 @@ How to use::
 Concretlty, it's such as simple as that::
 
     >>> from csv2oerp.callbacks import cb_method
-    >>> mapping = {
-    ...         'res.parter_address': {
+    >>> 'res.parter_address': {
     ...                 'partner_id': Column(25, callback=cb_method('res.partner', 'name'))
     ...             }
     ...     }
 
 
 """
-from constants_and_vars import STATS
-import unicode_csv
+from csv2oerp import unicode_csv
+from csv2oerp import tools
 from datetime import datetime
-import unicodedata
 import re
 
-def callback_model(self, model, field, line):
-    """Model of the calback signature.
+def callback_model(session, model, field, line):
+    u"""Model of the calback signature.
 
-    :param self: Current Session instance
-    :type self: Session
+    :param session: Current Session instance
+    :type session: Session
     :param model: Current model
     :type model: str
     :param field: Current field
@@ -69,8 +66,8 @@ def callback_model(self, model, field, line):
     """
     pass
 
-def get_line_from_table(csv_file, value, column, separator=',', quote='"', header=True):
-    """Search for an equivalence between value from column ``col_referer``
+def get_line_from_table(csv_file, value, column, separator=u',', quote=u'"', header=True):
+    u"""Search for an equivalence between value from column ``col_referer``
     and ``value`` value, and return value from column ``col_return`` index at
     the same line as matching pair ``col_referer`` values.
 
@@ -93,7 +90,7 @@ def get_line_from_table(csv_file, value, column, separator=',', quote='"', heade
     """
 
     res = []
-    with open(csv_file, 'r') as f:
+    with open(csv_file, u'r') as f:
         filereader = unicode_csv.Reader(f, delimiter=separator, quotechar=quote)
         for line in filereader:
             if header:
@@ -105,8 +102,8 @@ def get_line_from_table(csv_file, value, column, separator=',', quote='"', heade
     return res
 
 def search_table(csv_file, value, col_referer=0, col_return=1, 
-        separator=',', quote='"', header=True):
-    """Search for an equivalence between value from column ``col_referer``
+        separator=u',', quote=u'"', header=True):
+    u"""Search for an equivalence between value from column ``col_referer``
     and ``value`` value, and return value from column ``col_return`` index at
     the same line as matching pair ``col_referer`` values.
 
@@ -129,20 +126,19 @@ def search_table(csv_file, value, col_referer=0, col_return=1,
     """
 
     res = []
-    with open(csv_file, 'r') as f:
-        filereader = unicode_csv.Reader(f, delimiter=separator, quotechar=quote)
-        for item in filereader:
-            if header:
-                header = False
-                continue
-            if item[col_referer] == value:
-                res.append(item[col_return])
-    
+    filereader = unicode_csv.Reader(csv_file, delimiter=separator, quotechar=quote)
+    for item in filereader:
+        if header:
+            header = False
+            continue
+        if item[col_referer] == value:
+            res.append(item[col_return])
+
     return res
 
 def get_value_from_table(csv_file, col_referer=0, col_return=1,
-        separator=',', quote='"', header=True):
-    """Search for an equivalence between value from column ``col_referer`` index
+        separator=u',', quote=u'"', header=True):
+    u"""Search for an equivalence between value from column ``col_referer`` index
     and and value from ``args[3]``, and return value from column ``col_return``
     index at the same line as matching pair ``col_referer`` values.
     
@@ -167,19 +163,19 @@ def get_value_from_table(csv_file, col_referer=0, col_return=1,
     :returns: callback_model
 
     """
-    def _get_value_from_table(self, model, field, value, line):
+    def _get_value_from_table(session, model, field, value, line):
         return search_table(csv_file, value, col_referer, col_return,
                 separator, quote, header)
 
     return _get_value_from_table
 
-def get_ids(model, fields, op='&'):
-    """Return ``model`` which one or more ``fields`` matched column ``value``
+def get_ids(model, fields, op=u'&'):
+    u"""Return ``model`` which one or more ``fields`` matched column ``value``
    
     >>> 'partner_ids': Column(56, get_ids('res.partner.address', ['street', 'street2'], op='|'))
     [100, 102]
     
-    :param args: lambda args (ie: [self, model ,field, content, line])
+    :param args: lambda args (ie: [session, model ,field, content, line])
     :type args: list
     :param model: The name of the model to search into
     :type model: str
@@ -190,7 +186,7 @@ def get_ids(model, fields, op='&'):
     :returns: callback_model
 
     """
-    def _get_ids(self, b_model, b_field, b_value, b_line):
+    def _get_ids(session, b_model, b_field, b_value, b_line):
         search = []
         try:
             b_value = str(b_value).strip()
@@ -201,27 +197,26 @@ def get_ids(model, fields, op='&'):
             if i != len(fields):
                 search.append(op)
                 i += 1
-            search.append((field, '=', b_value))
+            search.append((field, u'=', b_value))
 
         try:
-            res = self.server.execute(self.db, self.uid, self.pwd,
-               model, 'search', search)
+            res = session.server.search(model, search)
         except RuntimeError:
-            return self._get_ids(b_model, b_field, b_value, b_line)
+            return _get_ids(b_model, b_field, b_value, b_line)
         if not res:
-            self._logger.error(u"<%-30s> '%s' not found in fields %s" %
-                    (model, b_value, fields), extra=STATS[self._id])
+            session._logger.error(u"<%20s> '%s' not found in fields %s" %
+                    (model, b_value, fields), extra=STATS[session._id])
         return res
     
     return _get_ids
 
-def get_id(model, fields, op='&'):
-    """Return ``model`` which one or more ``fields`` matched column ``value``
+def get_id(model, fields, op=u'&'):
+    u"""Return ``model`` which one or more ``fields`` matched column ``value``
    
     >>> 'address_ids': (21, get_ids('res.partner', ['name']))
     [99]
     
-    :param args: lambda args (ie: [self, model ,field, content, line])
+    :param args: lambda args (ie: [session, model ,field, content, line])
     :type args: list
     :param model: The name of the model to search into
     :type model: str
@@ -232,14 +227,14 @@ def get_id(model, fields, op='&'):
     :returns: callback_model
 
     """
-    def _get_id(*args):
+    def _get_id(*args): 
         res = get_ids(model, fields, op)(*args)
         return res and res[0] or False
     
     return _get_id
 
-def get_objects(model, fields, op='&'):
-    """Return ``model`` instance which ``fields``'s value matched ``value``
+def get_objects(model, fields, op=u'&'):
+    u"""Return ``model`` instance which ``fields``'s value matched ``value``
     
     >>> 'parent_name': Column(2, get_objects('res.partner.address', ['ref'])[0]['name'])
     ['Toto']
@@ -254,14 +249,14 @@ def get_objects(model, fields, op='&'):
 
     """
 
-    def _get_objects(self, b_model, b_field, b_value, b_line):
-        return self.server.execute(self.db, self.uid, self.pwd,
-               model, 'read', get_id(model, fields, op))
+    def _get_objects(session, b_model, b_field, b_value, b_line):
+        return session.server.execute(session.db, session.uid, session.pwd,
+               model, u'read', get_id(model, fields, op))
 
     return _get_objects
 
-def get_field(model, fields, field, op='&'):
-    """Return ``model``'s ``field`` field which ``fields``'s value matched ``value``
+def get_field(model, fields, field, op=u'&'):
+    u"""Return ``model``'s ``field`` field which ``fields``'s value matched ``value``
     
     >>> 'parent_name': Column(2, get_field('res.partner.address', ['ref'], 'name')
     ['Toto']
@@ -285,8 +280,8 @@ def get_field(model, fields, field, op='&'):
 
     return _get_field
 
-def get_fields(model, fields, op='&'):
-    """Return ``model`` instance which one or more ``fields`` matched column ``value``
+def get_fields(model, fields, op=u'&'):
+    u"""Return ``model`` instance which one or more ``fields`` matched column ``value``
     
     >>> 'parent_name': Column(2, get_fields('res.partner.address', ['ref']))
     'Toto'
@@ -301,15 +296,15 @@ def get_fields(model, fields, op='&'):
 
     """
 
-    def _get_fields(self, b_model, b_field, b_value, b_line):
-        ids = get_ids(model, fields, op)(self, b_model, b_field, b_value, b_line)
-        res = self.server.execute(self.db, self.uid, self.pwd, model, 'read', ids)
+    def _get_fields(session, b_model, b_field, b_value, b_line):
+        ids = get_ids(model, fields, op)(session, b_model, b_field, b_value, b_line)
+        res = session.server.execute(session.db, session.uid, session.pwd, model, u'read', ids)
         return res
 
     return _get_fields
 
-def to_boolean(true='1', false='0', none=''):
-    """Return a boolean depending from the input value.
+def to_boolean(true=u'1', false=u'0', none=u''):
+    u"""Return a boolean depending from the input value.
     
     >>> 'field': Column(2, to_boolean('Y', 'N', ''))
     
@@ -322,7 +317,7 @@ def to_boolean(true='1', false='0', none=''):
     :returns: callback_model
 
     """
-    def _to_boolean(self, model, field, value, line):
+    def _to_boolean(session, model, field, value, line):
         if value == str(true):
             return True
         elif value == str(false):
@@ -332,8 +327,8 @@ def to_boolean(true='1', false='0', none=''):
 
     return _to_boolean
 
-def to_date(formatted='%Y-%m-%d'):
-    """Check for a valid date and convert if needed before returning the value
+def to_date(formatted=u'%Y-%m-%d'):
+    u"""Check for a valid date and convert if needed before returning the value
 
     >>> 'inserted_date': Column(6, to_date())
     
@@ -342,26 +337,21 @@ def to_date(formatted='%Y-%m-%d'):
     :returns: callback_model
 
     """
-    def _to_date(self, model, field, value, line):
+    def _to_date(session, model, field, value, line):
+        date = value
         try:
             date = str(value.split('\n')[0])
         except:
             pass
-
-        try:
-            date = datetime.strptime(value, formatted)
-            res = date.strftime('%Y-%m-%d')
-        except Exception as err:
-            self._logger.error(u"Datetime malformed (%s)." % err,
-                    extra=STATS[self._id])
-            res = ''
-
-        return res
+        if date in (u'0000-00-00', u'0000/00/00', u'00-00-0000', u'00/00/0000', u''):
+            return u''
+        date = datetime.strptime(date, formatted)
+        return date.strftime(u'%Y-%m-%d')
 
     return _to_date
 
-def strip_accents(normalize='NFD', uce='Mn'):
-    """Return a accents stripped string
+def strip_accents(normalize=u'NFD', uce=u'Mn'):
+    u"""Return a accents stripped string
 
     .. note::
         
@@ -376,14 +366,13 @@ def strip_accents(normalize='NFD', uce='Mn'):
     :returns: callback_model
 
     """
-    def _strip_accents(self, model, field, value, line):
-        return ''.join((c for c in unicodedata.normalize(normalize, value)
-                    if unicodedata.category(c) != uce))
+    def _strip_accents(session, model, field, value, line):
+        return tools.string_accent(value, normalize, uce)
 
     return _strip_accents
 
 def city_cedex(res=None, cedex_code=False):
-    """Check if a cedex code is in the town name.
+    u"""Check if a cedex code is in the town name.
     Return the relative string to ``res`` argument
     
     >>> # initial value for column 6 = 'Paris Cedex 9'
@@ -397,9 +386,9 @@ def city_cedex(res=None, cedex_code=False):
     :returns: callback_model
 
     """
-    def _city_cedex(self, model, field, value, line):
+    def _city_cedex(session, model, field, value, line):
         city = value
-        value = unicode(strip_accents(value.lower()))
+        value = strip_accents(session, model, field, value.lower(), line)
         cedex = u""
 
         if value.count(u'cedex'):
@@ -409,17 +398,17 @@ def city_cedex(res=None, cedex_code=False):
             except Exception:
                 cedex = u"cedex"
         
-        if res == 'city':
+        if res == u'city':
             return city
-        elif res == 'cedex':
+        elif res == u'cedex':
             return cedex
         else:
             return (city, cedex)
 
     return _city_cedex
 
-def zfill(*args, **kwargs):
-    """Return a zfilled string.
+def zfill(*args):
+    u"""Return a zfilled string.
 
     .. note::
         
@@ -430,30 +419,13 @@ def zfill(*args, **kwargs):
     :returns: callback_model
     
     """
-    def _zfill(self, model ,field, value, line):
-        return value.zfill(*args, **kwargs)
+    def _zfill(session, model ,field, value, line):
+        return value.zfill(*args)
 
     return _zfill
 
-def lower(*args, **kwargs):
-    """Return a uppered string.
-
-    .. note::
-        
-        This function is a rewriting of string.lower, same signature
-
-    >>> 'address': Column(3, lower())
-    
-    :returns: callback_model
-    
-    """
-    def _upper(self, model ,field, value, line):
-        return value.lower(*args, **kwargs)
-
-    return _upper
-
-def upper(*args, **kwargs):
-    """Return a uppered string.
+def upper(*args):
+    u"""Return a uppered string.
 
     .. note::
         
@@ -464,13 +436,13 @@ def upper(*args, **kwargs):
     :returns: callback_model
     
     """
-    def _upper(self, model ,field, value, line):
-        return value.upper(*args, **kwargs)
+    def _upper(session, model ,field, value, line):
+        return value.upper(*args)
 
     return _upper
 
-def capitalize(*args, **kwargs):
-    """Return a capitalized string.
+def capitalize(*args):
+    u"""Return a capitalized string.
 
     .. note::
         
@@ -481,13 +453,13 @@ def capitalize(*args, **kwargs):
     :returns: callback_model
     
     """
-    def _capitalize(self, model ,field, value, line):
-        return value.capitalize(*args, **kwargs)
+    def _capitalize(session, model ,field, value, line):
+        return value.capitalize(*args)
 
     return _capitalize
 
-def strip(*args, **kwargs):
-    """Return a stripped string.
+def strip(*args):
+    u"""Return a stripped string.
 
     .. note::
         
@@ -498,19 +470,19 @@ def strip(*args, **kwargs):
     :returns: callback_model
     
     """
-    def _strip(self, model ,field, value, line):
-        return value.strip(*args, **kwargs)
+    def _strip(session, model ,field, value, line):
+        return value.strip(*args)
 
     return _strip
 
-def get_phones(type, prefix='+33'):
-    """Returns numbers contained in the columns values. Numbers must be in a 
-    french form such as `0123456789` or '33123456789' (No matters for spaces).
+def get_phones(type, prefix=u'+33'):
+    u"""Returns numbers contained in the columns values. Numbers must be in a 
+    french form such as `0123456789` or '+33123456789' (No matters for spaces).
     Any other numbers must not be in the columns values.
 
-    ..note:
+    ..note::
 
-        Example of a complex working string:
+        Example of a complex working string::
 
         "The number of Mr SMITH is 0123456789, his wife 33 234 567 894, and their son 06 24 568495"
 
@@ -520,13 +492,11 @@ def get_phones(type, prefix='+33'):
             'mobile': ['(+33) 6 24 56 84 95'], 
             'all': ['(+33) 1 23 45 67 89', '(+33) 2 34 56 78 94', '(+33) 6 24 56 84 95'], 
             'value': "The number of Mr SMITH is 02XXXXXXXX, his wife 33 2XX XXXXXX, and their son 06 XX XX XX XX XX"
-        }
-
-
+            }
     
-    >>> 'phone':  Column([3, 4], get_phones('phone', prefix='+33'))
-    >>> 'mobile': Column([3, 4], get_phones('mobile', prefix='+33'))
-    >>> 'phone': Column([3, 4], get_phones('all', prefix='+33'))
+        >>> 'phone':  Column([3, 4], get_phones('phone', prefix='+33'))
+        >>> 'mobile': Column([3, 4], get_phones('mobile', prefix='+33'))
+        >>> 'phone': Column([3, 4], get_phones('all', prefix='+33'))
     
     :param type: The format to returns
     :type type: str
@@ -536,16 +506,15 @@ def get_phones(type, prefix='+33'):
 
     """
     assert type
-    if type not in ('phone', 'mobile', 'all', 'dict'):
-        raise Exception('`type` is incorrect. Valid values are `phone`, `mobile`, `all` and `dict`')
+    if type not in (u'phone', u'mobile', u'all', u'dict'):
+        raise Exception(u'`type` is incorrect. Valid values are `phone`, `mobile`, `all`')
 
-    def _get_phones(self, model, field, value, line):
-        res = {'phone': [], 'mobile': [], 'all': []}
+    def _get_phones(session, model, field, value, line):
+        res = {u'phone': [], u'mobile': [], u'all': [], u'value': value}
 
         # On ne garde que les caractères numériques
         string = re.sub(u'[^0-9]', u'', value)
         stop = False
-        result = ''
         i = 0
         while not stop:
             i += 1
@@ -576,26 +545,26 @@ def get_phones(type, prefix='+33'):
             tel_list = [phone[0], phone[1:3], phone[3:5], phone[5:7], phone[7:9]]
 
             # Cas du téléphone portable
-            if tel_list[0] == '6':
-                res['mobile'].append((u'%s %s' % (prefix, u' '.join(tel_list))))
-                res['all'].append((u'%s %s' % (prefix, u' '.join(tel_list))))
+            if tel_list[0] == u'6':
+                res[u'mobile'].append((u'%s %s' % (prefix, u' '.join(tel_list))))
+                res[u'all'].append((u'%s %s' % (prefix, u' '.join(tel_list))))
             # Autre cas
             else:
-                res['phone'].append((u'%s %s' % (prefix, u' '.join(tel_list))))
-                res['all'].append((u'%s %s' % (prefix, u' '.join(tel_list))))
+                res[u'phone'].append((u'%s %s' % (prefix, u' '.join(tel_list))))
+                res[u'all'].append((u'%s %s' % (prefix, u' '.join(tel_list))))
 
             # Next loop
             string = string[9:]
 
-        if type is 'dict':
+        if type is u'dict':
             return res
 
         phones = res[type]
         if len(phones) > 1:
-            result = ' / '.join(phones)
+            result = u' / '.join(phones)
         elif len(phones) == 1:
             result = phones[0]
-        
+         
         return result
 
     return _get_phones
